@@ -5,7 +5,7 @@ package OpenAPI::Modern;
 # ABSTRACT: Validate HTTP requests and responses against an OpenAPI document
 # KEYWORDS: validation evaluation JSON Schema OpenAPI Swagger HTTP request response
 
-our $VERSION = '0.017';
+our $VERSION = '0.018';
 
 use 5.020;  # for fc, unicode_strings features
 use Moo;
@@ -271,20 +271,22 @@ sub _find_path ($self, $state, $request, $options) {
       if lc $request->method ne $method;
   }
 
-  my $uri_path = URI::Escape::uri_unescape($request->uri->path);
+  my $uri_path = $request->uri->path;
 
   if (not $path_template) {
     my $schema = $self->openapi_document->schema;
     croak 'servers not yet supported when matching request URIs'
       if exists $schema->{servers} and $schema->{servers}->@*;
+
     foreach $path_template (sort keys $schema->{paths}->%*) {
-      my $path_pattern = $path_template =~ s!\{[^/?#}]+\}!([^/]*)!gr;
+      my $path_pattern = $path_template =~ s!\{[^/}]+\}!([^/?#]*)!gr;
       next if $uri_path !~ m/^$path_pattern$/;
 
       # perldoc perlvar, @-: $n coincides with "substr $_, $-[n], $+[n] - $-[n]" if "$-[n]" is defined
-      my @capture_values = map substr($uri_path, $-[$_], $+[$_]-$-[$_]), 1 .. $#-;
+      my @capture_values = map
+        Encode::decode('UTF-8', URI::Escape::uri_unescape(substr($uri_path, $-[$_], $+[$_]-$-[$_]))), 1 .. $#-;
       my @capture_names = ($path_template =~ m!\{([^/?#}]+)\}!g);
-      my %path_captures; @path_captures{@capture_names} = @capture_values;
+      my %path_captures; @path_captures{@capture_names} = map URI::Escape::uri_unescape($_), @capture_values;
       return ($path_template, \%path_captures);
     }
 
@@ -292,20 +294,23 @@ sub _find_path ($self, $state, $request, $options) {
       'no match found for URI path "%s"', $uri_path);
   }
 
-  # note: we aren't doing anything special with escaped braces. this bit of the spec is hazy.
-  my @capture_names = ($path_template =~ m!\{([^/?#}]+)\}!g);
+  # note: we aren't doing anything special with escaped slashes. this bit of the spec is hazy.
+  my @capture_names = ($path_template =~ m!\{([^/}]+)\}!g);
   abort({ %$state, keyword => 'paths', _schema_path_suffix => $path_template },
       'provided path_captures names do not match path template "%s"', $path_template)
     if exists $options->{path_captures}
       and not is_equal([ sort keys $options->{path_captures}->%*], [ sort @capture_names ]);
 
-  my $path_pattern = $path_template =~ s!\{[^/?#}]+\}!([^/]*)!gr;
+  # 3.2: "The value for these path parameters MUST NOT contain any unescaped “generic syntax”
+  # characters described by [RFC3986]: forward slashes (/), question marks (?), or hashes (#)."
+  my $path_pattern = $path_template =~ s!\{[^/}]+\}!([^/?#]*)!gr;
   abort({ %$state, keyword => 'paths', _schema_path_suffix => $path_template },
       'provided %s does not match request URI', exists $options->{path_template} ? 'path_template' : 'operation_id')
     if $uri_path !~ m/^$path_pattern$/;
 
   # perldoc perlvar, @-: $n coincides with "substr $_, $-[n], $+[n] - $-[n]" if "$-[n]" is defined
-  my @capture_values = map substr($uri_path, $-[$_], $+[$_]-$-[$_]), 1 .. $#-;
+  my @capture_values = map
+    Encode::decode('UTF-8', URI::Escape::uri_unescape(substr($uri_path, $-[$_], $+[$_]-$-[$_]))), 1 .. $#-;
   abort({ %$state, keyword => 'paths', _schema_path_suffix => $path_template },
       'provided path_captures values do not match request URI')
     if exists $options->{path_captures}
@@ -315,7 +320,6 @@ sub _find_path ($self, $state, $request, $options) {
   return ($path_template, \%path_captures);
 }
 
-# for now, we only use captures, rather than parsing the URI directly.
 sub _validate_path_parameter ($self, $state, $param_obj, $path_captures) {
   # 'required' is always true for path parameters
   return E({ %$state, keyword => 'required' }, 'missing path parameter: %s', $param_obj->{name})
@@ -539,7 +543,7 @@ OpenAPI::Modern - Validate HTTP requests and responses against an OpenAPI docume
 
 =head1 VERSION
 
-version 0.017
+version 0.018
 
 =head1 SYNOPSIS
 
